#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <sys/stat.h>
#include <unistd.h>
#include "consts.h"

#include <ios>
#include <iostream>
#include <ostream>

/* ____________________________________________________________
             
              *** Global Constants ***
 */
#define MAX_LINE_LENGTH 2000 
#define MAX_WORD_LENGTH 600 //maximum length of a char word
#define MAX_FLAG_LENGTH 20 //maximum length of a flag-word
#define MSG_LENGTH 80 // maximum length of a short message
#define FLAGLEN 10 // length of a flag word
#define ACCURACY 1e-6 // accuracy of computations: in expressions of comparison
#define PI M_PI




/*  ___________________________________________________________ 
               
              ***  MACROSES ***
*/
#define dprint(expr) printf (#expr "=%e\n", (double)expr) ;fflush(NULL); 
#define sprint(expr) printf (#expr "=%s\n", expr) ;  fflush(NULL);
#define iprint(expr) printf (#expr "=%d\n", (int)expr); fflush(NULL);
#define dprin(expr) printf (#expr "=%e ", (double)expr) ;fflush(NULL); 
#define sprin(expr) printf (#expr "=%s ", expr) ;  fflush(NULL);
#define iprin(expr) printf (#expr "=%d ", (int)expr); fflush(NULL);

#define dpr(expr) fprintf (fout, "# " #expr "=%e\n", (double)expr) ; 
#define spr(expr) fprintf (fout, "# " #expr "=%s\n", expr) ; 
#define ipr(expr) fprintf (fout, "# " #expr "=%d\n", (int)expr);


#define sqr(x) ((x)*(x))
#define kub(x) ((x)*(x)*(x))
#define min(a,b) ((a) < (b) ? (a) : (b))
#define max(a,b) ((a) > (b) ? (a) : (b))
#define SWITCH(a,b,temp) (temp)=(a);(a)=(b);(b)=(temp);
#define Index(i,j) ((2*i)+j)  // i=ctrochka, j=stolbez
#define IndeX(i,j,Nc) ((Nc*i)+j)  // i=ctrochka(0...), j=stolbez (0...) Nc=chislo stolbzov


//Assign file named "exec_time.mark.endname" of some type and check it:
#define outfile(__filevar,__endname,__type) FILE *__filevar; \
   char __dir[12]; strcpy(__dir,"prog_output/");\
   if (strlen(__dir)+5+strlen(__endname)+strlen(mod.mark)+strlen(mod.exec_time)>MAX_WORD_LENGTH) {printf("File name with %s is too long.\n", __endname); err_m(ERRORPOINT);}\
   sprintf(__filename,"prog_output/%s.%s.%s",mod.exec_time,mod.mark,__endname);\
   if (!(__filevar=fopen(__filename,__type))) {printf(" Cannot open file %s\n", __filename); err_m(ERRORPOINT);}\
   if (strlen(__outfilenames)+strlen(__filename)<MAX_WORD_LENGTH*10){\
      sprintf(__outfilenames,"%s%s (%s);\n",__outfilenames,__filename, __type);}

#define file(__filevar,__name,__type)     FILE *__filevar; \
   if (strlen(__name)>MAX_WORD_LENGTH) {printf("File name is too long: %s\n", __name); err_m(ERRORPOINT);}\
   strcpy(__filename,__name);\
   if (!(__filevar=fopen(__filename,__type))) {printf(" Cannot open file %s\n", __name); err_m(ERRORPOINT);}\
   if (strlen(__outfilenames)+strlen(__filename)+strlen(__type)+5<MAX_WORD_LENGTH*10) {\
   sprintf(__outfilenames,"%s%s (%s);\n",__outfilenames,__filename, __type);}

// Assign memory for dynamic array "arr" of size s:REMEMBER To DELETE AFTER USE
#define makeda(arr,s) if (s<=0) {printf ("anounced size=%d: ", s); err_m(2);}\
                      double *arr = new double[(s)]; if (!arr) { err_m(1); }
      
                       
// Assign memory for dynamic 2D array "arr" of size [s1][s2]: REMEMBER To DELETE AFTER USE		       
#define maked2a(arr,s1,s2)  double **arr = new double*[(s1)]; \
                            if (!arr) { err_m(1); }           \
                            for (long v=0; v<(s1); v++) {     \
                               arr[v] = new double[(s2)];     \
                               if (!arr[v]) { err_m(1); }     \
			    }
// Assign memory for dynamic 3D array "arr" of size [s1][s2][s3]:REMEMBER To DELETE AFTER USE		       
#define maked3a(arr,s1,s2,s3)  double ***arr = new double**[(s1)]; \
                            if (!arr) { err_m(1); }           \
                            for (long v=0; v<(s1); v++) {     \
                               arr[v] = new double*[(s2)];     \
                               if (!arr[v]) { err_m(1); }     \
                               for (long u=0; u<(s2); u++) {     \
                                  arr[v][u] = new double[(s3)];     \
                                  if (!arr[v][u]) { err_m(1); }     \
			         } \
			    }

//Free memory from dynamic 1D array "arr" :	
#define deleteda(arr)    delete [] arr; \
	                 arr=NULL;
//Free memory from dynamic 2D array "arr" of size [s1][s2]:	
#define deleted2a(arr,s1) for (long v=0; v<(s1); v++) {   \
			     delete [] arr[v];            \
			  }                               \
			  delete [] arr;                  \
			  arr=NULL;
                            
//Free memory from dynamic 3D array "arr" of size [s1][s2][s3]:	
#define deleted3a(arr,s1,s2) for (long v=0; v<(s1); v++) {  \
                               for (long u=0; u<(s2); u++) {\
			         delete [] arr[v][u];      \
                               }                           \
                               delete [] arr[v];            \
			  }                               \
			  delete [] arr;                  \
			  arr=NULL;

#define delete2(arr)  delete [] (arr)

/*  ___________________________________________________________ 
               
            ***  Routines of general utilization ***
*/

int  check_odd(int value); // returns 1, if value is odd; -1, if even
void choice_min_max(double *);

double last_stable_orbit(double _mass, double _kerr); // cm, in Kerr regime

double fmaximum (double (*fun)(double, struct model),
   double x_left, double x_right, double *x_max, struct model mod);
double fmaximum1 (double (*fun)(double, double, struct model),
   double x_left, double x_right, double *x_max, double some_par, struct model mod);   


double sumvec(double y[], int num_points); // sum of vector's elements
double integral_simpson(double x[], double y[], int num_points, int flag);
double integral_simpson_interval ( double x[], double y[],
	   int x_volume, int flag, double x_low, double x_up,
	   int num_points, int check_N_low, int check_N_up );
double integrate(double *x, double *y, int num_points, int flag, 
int int_points, double x_min, double x_max);	   

void spline(double x[], double y[], int n, double yp1, double ypn, 
   double y2[]);
double splint(double xa[], double ya[], double y2a[], int n, double x, 
   double *y);

void make_mesh( double x_lim1, double x_lim2, int num_points, int flag,
	   double *x); /* Make array x[] with  num_points
	   x-values in limits between x_lim1 and x_lim2, inclusive */
	
void err_m (int i);  /* Print error message and exit(i) */	
int get_runtime(char *runtime); /* returns integer seconds of system time
and "runtime" - a string with the whole time */
void  rdcols (char *fname, int colnum, double *vector, int *size); 
   /* read columns with double values from a file with name "fname";
   */
void print_col_nums(int colnum, FILE *fout); 
int read_table(double ** _data, char *_fname, int _ncols, int _nrows);   
   /* print column numbers in a file with result */

/*  ___________________________________________________________ 
               
            ***  Variables for general utilization ***
*/

				     
#ifdef _MAIN_
char __filename[MAX_WORD_LENGTH]; // variable with a file name
char __outfilenames[MAX_WORD_LENGTH*10]; // variable with all output file names
int verbose; // verbosity level

// =========================================================================
// =========================================================================
// =========================================================================

#else
extern char __filename[MAX_WORD_LENGTH]; // variable with a file name
extern char __outfilenames[MAX_WORD_LENGTH*10]; // variable with all output file names
extern int verbose; // verbosity level
#endif

// =========================================================================



