#include "head.v3.h"
#include "gsl_sf_bessel.h"
#include <gsl/gsl_spline.h> 
#include <gsl/gsl_fit.h>
//#include <gsl/gsl_math.h>
#include <gsl/gsl_deriv.h>
#include <gsl/gsl_integration.h>

#define ACCUX 0.0001
#define NUM_COOR  99 // 90, 559
#define FITSIZE 5  

void print_mem(void const *vp, size_t n);

//main.c:
void evol_mdot_in (struct gmodel * mod);
double F_x_t (double x, double t, struct gmodel * mod ) ;
void F_init_disk(struct gmodel * mod, int nbins, 
      double *g_init, double *x1);
void F_1maximum_disk(struct gmodel * mod, int nbins, double *zero_inter,
      double *x1);
void get_initial_from_file (int len, double *zero_interp, double *x1);
double mod_Green_h_x1_t (double h, void * p);
double Green_x_x1_t (double x, double t,double x1,struct gmodel * mod);
double mdot_in_t (double t, double * abserr_mdot,struct gmodel * mod );
double numeric_mdot_Green_x_x1_t (double x, double t,double x1,
   struct gmodel * mod, double * abserr );
double mdot_Green_x_x1_t (double x, double t,double x1,
   struct gmodel * mod);
double F_h_t (double h, void * p) ;


void save_F_t ( double t, struct gmodel * mod, int ti);
void set_initial_F (struct gmodel * mod);

void get_zeroes(int size, double order, double *zeroes, double *l1, double *l2);

double integrant (double x1, void * p);
double F_initial (double x, struct gmodel * mod);
double interp_initial_F (double x1, struct gmodel * mod);
double sigma_init_r(double r , struct gmodel *mod );

void g_rised_alpha (struct gmodel * mod, int nbins, double *g_init,
      double *x1) ;


void save_mdotgreen_t( double t, struct gmodel * mod, int ti);
void write_model (FILE * OUT, struct gmodel * mod) ;      
void read_parameters (FILE *  IN, struct gmodel * mod) ;


double integrant_green_over_t (double tau, void * p);
double  integral_green_over_t(double x, double x1, double t, 
   struct gmodel * mod, double source_val);
double  integral_green_over_t_analyt (double x, double x1, double t, 
   struct gmodel * mod, double source_val);
double get_value_integral_green_over_tau(double x, double x1, double t, 
   struct gmodel * mod, double source_val) ;

double mdot_in_t1 (double t, double * abserr_mdot,
   struct gmodel * mod );



double get_value_integral_vari_accrate_over_t( double x, double x1,
   double t, struct gmodel * mod);
double  integral_vari_accrate_over_t (double x, double x1, double t, 
   struct gmodel * mod ) ;
double source_fun (double x1,  double tau, struct gmodel *mod) ;
double mdot_out_m94 (double tau); 
double d_mdot_out_d_t_m94(double tau);



double calc_time_int_i(double t, int i, struct gmodel  * mod); 
double sum_time_int (double x, double t, struct gmodel * mod);
double integrant_tii (double tau, void * p);

void save_inner_F (double t1, struct gmodel *mod);


double mdot_out_const (double tau);
double deriv_mdot_out_zero (double tau);


void save_F_t_cube ( double t, struct gmodel * mod, double mdot, 
       int * flag, double tshift);
      
struct gmodel {
   double mass;  //mass of central object, gramm
   double mdisk; // mass of the accretion disk
   double mdot_out ; // constant Mdot at Rout, g/s (transfer from secondary)
   double r_out_disk; // outer disk radius in the physical model, cm
   double r_out_vis;
   double l ; // order of Bessel functions in the solution
   int len; // humber of terms in sum determining Bessel function
   double a; // outer radius for solution
   double kappa; 
   double nu0 ;
   double texp;
   double zeroes[200]; // zeroes of bessel fucntion
   double constl1[200]; // pre-calculated constants depending on Lommel integrals
   double constl2[200]; // more pre-calculated constants depending on Lommel integrals
    int flag_initial; // how to determine initial g(x1,t)
   int flag_outer; // 0 - Mout=0; 1 - Mout=const
   int calcgflag;  // what terms  to  sum in series of Green's function
   int have_integral;
   int flag_green;  // household flag to calculate green function
   double init_disk_part; // part of disk in radii evolving at t=0 
   double tmax1; // time of 1st maximum
   double r_ign_in; 
   double r_ign_out;
   double r_mass_out;  // outer radius  of the Mdisk location
   double r_mass_in; // innder radius  of the Mdisk location
   double sigma_index; // power - law index for sigma(r) distribution
   double zero_interp[NUM_COOR];
   double green_int_over_t_x[NUM_COOR];
   double F_stage_one[NUM_COOR];
   double rise_alpha_disk_part  ; // part where alpha is risen due to convection
   double alpha_rise; // factor to rise alpha
   double (* accr_out) (double tau); // variable Mdot at Rout, g/s (transfer from secondary)
   double (* d_accr_out_d_t) (double tau); // time derivative of Mdot at Rout, g/s (transfer from secondary) 
};


struct my_Green_params { double t;double x1;struct gmodel  * mod ; };
struct my_Green_params1 { double t; double x1; double x; 
                        struct gmodel *mod; double source_val ;};
struct my_Green_params2 { double t; struct gmodel  * mod ; };
struct my_Green_params3 { double t; double x1; double x; 
                        struct gmodel *mod;          
                        };
struct my_Green_params4 { double t; int i; struct gmodel  * mod ; };                        

                        
